# frozen_string_literal: true

require 'apple_core/refinements/string'

module  Apiphobic
module  Authorization
module  Authorizers
class   Scope
  using ::AppleCore::Refinements::String

  attr_accessor :action,
                :audience,
                :raw_parameters,
                :scope_root,
                :token

  # rubocop:disable Metrics/ParameterLists
  def initialize(action:, token:, audience:, issuer:, parameters:, scope_root:, **other)
    self.action         = action
    self.audience       = audience
    self.raw_parameters = parameters
    self.scope_root     = scope_root
    self.token          = token

    other.each do |name, value|
      public_send("#{name}=", value)
    end
  end
  # rubocop:enable Metrics/ParameterLists

  alias user audience

  def user_scope
    scope_root.public_send("for_#{user_underscored_class_name}", scope_user_id)
  end

  def public_scope
    scope_root.none
  end

  def call
    if user.nil?
      public_scope
    else
      user_scope
    end
  end

  private

  def scope_user_id
    authorized_scope_id(name: user_underscored_class_name, default: user.id)
  end

  def user_underscored_class_name
    @user_underscored_class_name ||= begin
      base_user_class_name         = user.class.name[/([^:]+)\z/, 1]

      base_user_class_name.underscore.downcase
    end
  end

  def authorized_scope_id(name:, default:)
    if token.admin?
      raw_parameters
        .fetch(:filter, {})
        .fetch("#{name}_id", default)
    else
      default
    end
  end
end
end
end
end
