# frozen_string_literal: true

require 'apiphobic/json_api/relationship'

module  Apiphobic
module  Authorization
module  Transformers
class   JsonApiToRailsAttributes
  attr_accessor :parameters

  def initialize(parameters:)
    self.parameters = parameters
  end

  def call
    attributes.merge(relationship_attributes)
  end

  private

  def attributes
    @attributes = parameters
                    .fetch(:data,       {})
                    .fetch(:attributes, parameters_class.new)
  end

  def relationships
    @relationships = parameters
                       .fetch(:data,          {})
                       .fetch(:relationships, parameters_class.new)
  end

  def relationship_attributes
    parameters_class.new.tap do |relationship_attributes|
      relationships.each_pair do |name, relationship|
        relationship_attributes.merge!(JsonApi::Relationship
                                         .new(name: name, data: relationship)
                                         .to_rails_attributes)
      end

      relationship_attributes.permit! if relationship_attributes.respond_to?(:permit!)
    end
  end

  def parameters_class
    parameters.class
  end
end
end
end
end
